#!perl -w
package ini_file;
use strict;
use IO::File;
use Tie::IxHash;
sub new {
	my $class = shift;
	my ($fn, $mode) = @_;

	my $fh = IO::File->new($fn, $mode) or return undef;

	my $self = {};
	$self->{fh} = $fh;
	$self->{sections_list} = [];
	$self->{sections_hash} = ();
	
	bless($self, $class);

	$mode eq 'w' and return $self;

	my $section;
	my $skip_section = 0;
	while (<$fh>) {
		$_ =~ qr/^\s*;/ and next;
		if (/^\[(.*)\]\s*:*\s*(\w.*)?/) {
			if (defined $2 && !::is_flag_defined($2)) {
				$skip_section = 1;
				next;
			}
			$section = $1;
			die if defined $self->{sections_hash}->{$section};
			push @{$self->{sections_list}}, $section;
			my %tmp = ();
			$self->{sections_hash}{$section} = \%tmp;
			tie (%{$self->{sections_hash}{$section}}, "Tie::IxHash");
			$skip_section = 0;
			next;
		}
		if (/^([^=]*?)\s*=\s*(.*?)\s*$/) {
			my ($name, $value) = ($1, $2);
			if ($value =~ /^<<(.+)$/) {
				my $terminator = $1;
				$value = '';
				while (<$fh>) {
					chomp;
					last if $_ eq $terminator;
					$value .= "\n".$_;
				}
				die unless defined $_;
				substr ($value, 0, 1) = '';
			}
			$skip_section == 1 and next;
			die unless defined $section;
			$self->{sections_hash}{$section}{$name} = $value;
		}
	}
	return $self;
}
sub save {
	my $self = shift;
	my ($fn) = @_;
	my $fh = IO::File->new($fn, 'w');
	foreach my $section (sort {$a <=> $b} @{$self->{sections_list}}) {
		print $fh "[$section]\n";
		foreach my $param (keys %{$self->{sections_hash}{$section}}) {
			if ($param eq 'custom_data') {
				print $fh "custom_data = <<END\n";
				print $fh "$self->{sections_hash}{$section}{$param}\n";
				print $fh "END\n";
			} else {
				print $fh "$param = $self->{sections_hash}{$section}{$param}\n";
			}
		}
		print $fh "\n";
	}
	$fh->close();
}
sub close {
	my $self = shift;
	$self->{fh}->close();
	$self->{fh} = undef;
}
#################################################
package main;
use strict;

my $fh = ini_file->new($ARGV[0], 'r');
my $ini = ini_file->new('config.ini', 'r');
if (defined $ini->{sections_hash}{'sections_exchange'}{'indexes'}) {
	my @indexes = split /,/, $ini->{sections_hash}{'sections_exchange'}{'indexes'};
	foreach my $index (@indexes) {
		delete $fh->{sections_hash}{$index};
		$fh->{sections_hash}{$index} = $ini->{sections_hash}{$index};
	}
}
if (defined $ini->{sections_hash}{'parameters_exchange'}) {
	foreach my $name (keys %{$ini->{sections_hash}{'parameters_exchange'}}) {
		my $value = $ini->{sections_hash}{'parameters_exchange'}{$name};
		foreach my $section (keys %{$fh->{sections_hash}}) {
			$fh->{sections_hash}{$section}{$name} = $value;
		}		
	}
}
if (defined $ini->{sections_hash}{'values_exchange'}) {
	foreach my $section (keys %{$fh->{sections_hash}}) {
		foreach my $param (keys %{$fh->{sections_hash}{$section}}) {
			my $value = $fh->{sections_hash}{$section}{$param};
			if ($value =~ /\w+,/) {
				my @temp = split /,/, $value;
				$temp[0] = $ini->{sections_hash}{'values_exchange'}{$temp[0]} if defined $ini->{sections_hash}{'values_exchange'}{$temp[0]};
				$value = join ',', @temp;
				$fh->{sections_hash}{$section}{$param} = $value;
			}
			$fh->{sections_hash}{$section}{$param} = $ini->{sections_hash}{'values_exchange'}{$value} if defined $ini->{sections_hash}{'values_exchange'}{$value};
		}
	}		
}
$fh->save($ARGV[0].'.new');
$fh->close();
$ini->close();
#################################################